/*
 * Copyright (c) 2005-2010 Imperas Software Ltd., www.imperas.com
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied.
 *
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
#include "svdpi.h"
#include "dpiheader.h"

#include "icm/icmCpuManager.h"

//typedef struct extMemDescS {
//    void *localSource;
//    void *localSink;
//} extMemDesc;

static ICM_MEM_WRITE_FN(extMemWriteCB) {

    bfm_write((int)address,*(int *)value);
    
    //extMemDesc *p;
    //memcpy(p->localSink, value, bytes);

    //icmPrintf(
    //    "EXTERNAL MEMORY: Writing 0x%08x 0x%08x\n",
    //     (Int32)address, *(Int32 *)value
    //);
}

static ICM_MEM_READ_FN(extMemReadCB) {

    int o;
    bfm_read((int)address,&o);
    *(Int32 *)value = o;

    //extMemDesc *p = userData;
    //memcpy(value, p->localSource, bytes);

    //icmPrintf(
    //    "EXTERNAL MEMORY: Reading 0x%08x 0x%08x\n",
    //    (Int32)address, *(Int32 *)value
    //);
}


static ICM_MEM_WRITE_FN(watchWriteCB) {

    icmPrintf(
        "WATCHPOINT '%s': 0x%08x 0x%08x\n",
	(Uns8 *)userData,
        (Int32)address,
        *(Int32 *)value
    );
}

static ICM_MEM_READ_FN(watchReadCB) {

    icmPrintf(
        "WATCHPOINT '%s': 0x%08x 0x%08x\n",
        (Uns8 *)userData,
        (Int32)address,
	*(Int32 *)value
    );
}


icmProcessorP createPlatform(void) {

////////////////////////////////////////////////////////////////////////////////

    icmInit(ICM_VERBOSE|ICM_STOP_ON_CTRLC, 0, 0);

    icmSetPlatformName("BareMetalPowerpc32Single");


////////////////////////////////////////////////////////////////////////////////
//                                 Bus bus                                 //
////////////////////////////////////////////////////////////////////////////////

    icmBusP bus = icmNewBus( "bus", 32);


////////////////////////////////////////////////////////////////////////////////
//                               Processor cpu1                               //
////////////////////////////////////////////////////////////////////////////////
#define SIM_ATTRS (ICM_ATTR_RELAXED_SCHED)

    icmAttrListP cpu1_attr = icmNewAttrList();

    icmAddStringAttr(cpu1_attr, "compatibility", "isa");
    icmAddStringAttr(cpu1_attr, "endian",        "big");
    icmAddDoubleAttr(cpu1_attr, "mips", 100.000000);

    // select library components
    const char *powerpc32Model    = icmGetVlnvString(NULL, "power.ovpworld.org", "processor",  "powerpc32",        "1.0", "model");
    const char *powerpc32Semihost = icmGetVlnvString(NULL, "power.ovpworld.org", "semihosting", "powerpc32Newlib", "1.0", "model");

    // create a processor
    icmProcessorP cpu1_c = icmNewProcessor(
        "cpu1",              // name
        "powerpc32",         // type
        0,                   // cpuId
        0x0,                 // flags
        32,                  // address bits
        powerpc32Model,      // model
        "modelAttrs",        // symbol
        SIM_ATTRS,           // procAttrs
        cpu1_attr,           // attrlist
        powerpc32Semihost,   // semihost file
        "modelAttrs"         // semihost symbol
    );

    icmConnectProcessorBusses( cpu1_c, bus, bus );


////////////////////////////////////////////////////////////////////////////////
//                                Memory memory                               //
////////////////////////////////////////////////////////////////////////////////
    // create two simulated memories for low and high regions
    icmMemoryP memory1 = icmNewMemory("mem1", ICM_PRIV_RWX, 0xffffff8f);
    icmMemoryP memory2 = icmNewMemory("mem2", ICM_PRIV_RWX, 0xffffffff-0xffffffa0);

    //static extMemDesc extMem;

    // map the address range 0x00400000:0x00400fff externally to the processor,
    icmMapExternalMemory(
        bus, "external", 7, 0xffffff90, 0xffffff9f,
        extMemReadCB, extMemWriteCB, 0 //&extMem
    );

    // connect memories to bus
    icmConnectMemoryToBus(bus, "mp1", memory1, 0);
    icmConnectMemoryToBus(bus, "mp2", memory2, 0xffffffa0);

    // show the bus connections
    icmPrintBusConnections(bus);
    
    return cpu1_c;
}

////////////////////////////////////////////////////////////////////////////////
//                                   M A I N                                  //
////////////////////////////////////////////////////////////////////////////////

int main() {

    //const char *appName = "asmtest.OR1K.elf";
    const char *appName = "fibonacci.POWERPC32.elf";

    // the constructor
    icmProcessorP processor = createPlatform();

    // Create a watchpoint
    // Invoke callback on write accesses to the address 0x00400000-0x00400003
    icmAddWriteCallback(
        processor,          // processor
        0xffffff90,         // low address
        0xffffff9f,         // high address
        watchWriteCB,       // callback to invoke
        "write     "     // user data passed to callback
    );
    // Create a watchpoint
    // Invoke callback on read accesses to the address 0x00400000-0x00400003
    icmAddReadCallback(
        processor,          // processor
        0xffffff90,         // low address
        0xffffff9f,         // high address
        watchReadCB,        // callback to invoke
        "read      "     // user data passed to callback
    );

    // load the application executable file into processor memory space
    if(!icmLoadProcessorMemory(processor, appName, False, False, True)) {
        return -1;
    }

    icmPrintf("\nStarting Simulation ...\n");

    // run till the end
    icmSimulatePlatform();

/*  // step with disassembling
    Bool done = False;

    while(!done) {

        Uns32 currentPC = (Uns32)icmGetPC(processor);

        // disassemble instruction at current PC
        icmPrintf(
            "0x%08x : %s\n", currentPC,
            icmDisassemble(processor, currentPC)
        );

        // execute one instruction
        done = (icmSimulate(processor, 1) != ICM_SR_SCHED);

    }
*/
    icmPrintf("\n");

    // free simulation data structures
    icmTerminate();

    return 0;
}

////////////////////////////////////////////////////////////////////////////////
