module bfm_ovp(
                nCS,
                nRD,
                nWR,
                HA,
                HD
                );

    // parameter definition
    parameter CPU_SIM_ACC = 1'b0;      // accelerate the CPU_BFM simulation speed
    //input output
    output              nCS;
    output              nRD;
    output              nWR;
    output [31:0]       HA;
    inout  [31:0]       HD;
    
    reg              nCS;
    reg              nRD;
    reg              nWR;
    reg [31:0]       HA;
    
    //inout interface
    reg  [31:0]         HD_i;
    reg  [31:0]         HD_o;
    reg                 HD_en;
    //inout interface
    assign  HD = (HD_en==1'b1)?HD_o:31'bz;

    //write
    reg [31:0] wr_addr;
    reg [31:0] wr_data;
    export "DPI-C" task bfm_write;
    task bfm_write(input int addr, input int data);
    begin
        $display("SystemVerilog bfm_write: 0x%h 0x%h",addr,data);
	wr_addr = addr;
	wr_data = data;

        nCS = 1'b1;
        nRD    = 1'b1;
        nWR    = 1'b1;
        HA     = 31'd0;
        HD_en  = 1'b0;
        HD_o   = 31'd0;

        if (CPU_SIM_ACC)  #1; else #10;        // any
        HA = wr_addr[31:0];
        if (CPU_SIM_ACC)  #0; else #0;         // #10;
        HD_en = 1'b1;
        HD_o = wr_data[31:0];
        nCS = 1'b0;
        if (CPU_SIM_ACC)  #1; else #25;        // #40;
        nWR = 1'b0;
        if (CPU_SIM_ACC)  #1; else #90;        // #140;
        nWR = 1'b1;
        if (CPU_SIM_ACC)  #1; else #40;        // #40;
        nCS = 1'b1;
        HA = 31'b0;
        HD_o = 31'b0;
        HD_en = 1'b0;
        if (CPU_SIM_ACC)  #1; else #5;         // any
    end
    endtask

    //read
    reg [31:0] rd_addr;
    export "DPI-C" task bfm_read;
    task bfm_read(input int addr, output int data);
    begin
	rd_addr = addr;

        nCS = 1'b1;
        nRD = 1'b1;
        nWR = 1'b1;
        HA   = 31'b0;
        HD_en  = 1'b0;
        HD_o = 31'b0;

        if (CPU_SIM_ACC)  #1; else #10;
        HA = rd_addr[31:0];                    // #20;
        nCS = 1'b0;
        if (CPU_SIM_ACC)  #1; else #25;        // #30;
        nRD = 1'b0;
        if (CPU_SIM_ACC)  #1; else #160;       // #180;
	HD_i = HD;
        if (CPU_SIM_ACC)  #1; else #20;       // #180;
        nRD = 1'b1;
        if (CPU_SIM_ACC)  #1; else #30;        // #40;
        nCS = 1'b1;
        HA = 31'b0;
        if (CPU_SIM_ACC)  #1; else #5;

	data = HD_i;
        $display("SystemVerilog bfm_read : 0x%h 0x%h",addr,data);
    end
    endtask

    import "DPI-C" context task main();
    initial
    begin
        main();
    end
endmodule
